import { AstroTelemetry } from "@astrojs/telemetry";
import * as colors from "kleur/colors";
import yargs from "yargs-parser";
import { z } from "zod";
import * as event from "../events/index.js";
import add from "../core/add/index.js";
import build from "../core/build/index.js";
import { openConfig } from "../core/config.js";
import devServer from "../core/dev/index.js";
import { enableVerboseLogging, nodeLogDestination } from "../core/logger/node.js";
import { formatConfigErrorMessage, formatErrorMessage, printHelp } from "../core/messages.js";
import preview from "../core/preview/index.js";
import { createSafeError } from "../core/util.js";
import { check } from "./check.js";
import { openInBrowser } from "./open.js";
import * as telemetryHandler from "./telemetry.js";
function printAstroHelp() {
  printHelp({
    commandName: "astro",
    usage: "[command] [...flags]",
    headline: "Futuristic web development tool.",
    tables: {
      Commands: [
        ["add", "Add an integration."],
        ["build", "Build your project and write it to disk."],
        ["check", "Check your project for errors."],
        ["dev", "Start the development server."],
        ["docs", "Open documentation in your web browser."],
        ["preview", "Preview your build locally."],
        ["telemetry", "Configure telemetry settings."]
      ],
      "Global Flags": [
        ["--config <path>", "Specify your config file."],
        ["--root <path>", "Specify your project root folder."],
        ["--verbose", "Enable verbose logging."],
        ["--silent", "Disable all logging."],
        ["--version", "Show the version number and exit."],
        ["--help", "Show this help message."]
      ]
    }
  });
}
const ASTRO_VERSION = "1.0.0-beta.58";
async function printVersion() {
  console.log();
  console.log(`  ${colors.bgGreen(colors.black(` astro `))} ${colors.green(`v${ASTRO_VERSION}`)}`);
}
function resolveCommand(flags) {
  const cmd = flags._[2];
  if (cmd === "add")
    return "add";
  if (cmd === "telemetry")
    return "telemetry";
  if (flags.version)
    return "version";
  else if (flags.help)
    return "help";
  const supportedCommands = /* @__PURE__ */ new Set(["dev", "build", "preview", "check", "docs"]);
  if (supportedCommands.has(cmd)) {
    return cmd;
  }
  return "help";
}
async function cli(args) {
  var _a;
  const flags = yargs(args);
  const cmd = resolveCommand(flags);
  const root = flags.root;
  switch (cmd) {
    case "help":
      printAstroHelp();
      return process.exit(0);
    case "version":
      await printVersion();
      return process.exit(0);
  }
  let logging = {
    dest: nodeLogDestination,
    level: "info"
  };
  if (flags.verbose) {
    logging.level = "debug";
    enableVerboseLogging();
  } else if (flags.silent) {
    logging.level = "silent";
  }
  const telemetry = new AstroTelemetry({ version: ASTRO_VERSION });
  switch (cmd) {
    case "add": {
      try {
        telemetry.record(event.eventCliSession({ cliCommand: cmd }));
        const packages = flags._.slice(3);
        return await add(packages, { cwd: root, flags, logging, telemetry });
      } catch (err) {
        return throwAndExit(err);
      }
    }
    case "docs": {
      try {
        telemetry.record(event.eventCliSession({ cliCommand: cmd }));
        return await openInBrowser("https://docs.astro.build/");
      } catch (err) {
        return throwAndExit(err);
      }
    }
    case "telemetry": {
      try {
        const subcommand = (_a = flags._[3]) == null ? void 0 : _a.toString();
        return await telemetryHandler.update(subcommand, { flags, telemetry });
      } catch (err) {
        return throwAndExit(err);
      }
    }
  }
  const { astroConfig, userConfig } = await openConfig({ cwd: root, flags, cmd });
  telemetry.record(event.eventCliSession({ cliCommand: cmd }, userConfig, flags));
  switch (cmd) {
    case "dev": {
      try {
        await devServer(astroConfig, { logging, telemetry });
        return await new Promise(() => {
        });
      } catch (err) {
        return throwAndExit(err);
      }
    }
    case "build": {
      try {
        return await build(astroConfig, { logging, telemetry });
      } catch (err) {
        return throwAndExit(err);
      }
    }
    case "check": {
      const ret = await check(astroConfig);
      return process.exit(ret);
    }
    case "preview": {
      try {
        const server = await preview(astroConfig, { logging, telemetry });
        return await server.closed();
      } catch (err) {
        return throwAndExit(err);
      }
    }
  }
  throwAndExit(new Error(`Error running ${cmd} -- no command found.`));
}
function throwAndExit(err) {
  if (err instanceof z.ZodError) {
    console.error(formatConfigErrorMessage(err));
  } else {
    console.error(formatErrorMessage(createSafeError(err)));
  }
  process.exit(1);
}
export {
  cli
};
